@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2013 Free Software Foundation, Inc.
+ * Copyright (C) 2002-2014 Free Software Foundation, Inc.
  *
  * This file is part of LIBTASN1.
  *
@@ -44,7 +44,7 @@ extern "C"
 {
 #endif
 
-#define ASN1_VERSION "3.3"
+#define ASN1_VERSION "3.5"
 
   /*****************************************/
   /* Errors returned by libtasn1 functions */
@@ -168,6 +168,12 @@ extern "C"
 #define ASN1_ETYPE_UTC_TIME       36
 #define ASN1_ETYPE_GENERALIZED_TIME 37
 
+/* Flags used by asn1_delete_structure2() */
+
+/* makes sure the values are zeroized prior to deinitialization */
+#define ASN1_DELETE_FLAG_ZEROIZE 1
+
+
   struct asn1_data_node_st
   {
     const char *name;		/* Node name */
@@ -214,6 +220,8 @@ extern "C"
 
   extern ASN1_API int asn1_delete_structure (asn1_node * structure);
 
+  extern ASN1_API int asn1_delete_structure2 (asn1_node * structure, unsigned int flags);
+
   extern ASN1_API int
     asn1_delete_element (asn1_node structure, const char *element_name);
 
