@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2013 Free Software Foundation, Inc.
+ * Copyright (C) 2002-2014 Free Software Foundation, Inc.
  *
  * This file is part of LIBTASN1.
  *
@@ -24,6 +24,7 @@
 /* Description: Test sequences for these functions:  */
 /*     asn1_visit_tree,                              */
 /*     asn1_create_element,                          */
+/*     asn1_delete_structure2,                       */
 /*     asn1_delete_structure,                        */
 /*     asn1_write_value,                             */
 /*     asn1_read_value,                              */
@@ -58,6 +59,7 @@
 #define ACT_READ_BIT           19
 #define ACT_SET_DER            20
 #define ACT_DELETE_ELEMENT     21
+#define ACT_DELETE2	22
 
 
 typedef struct
@@ -82,12 +84,13 @@ test_type test_array[] = {
   {ACT_ENCODING, "", 0, 4, ASN1_MEM_ERROR, __LINE__},
   {ACT_ENCODING, "", 0, 5, ASN1_SUCCESS, __LINE__},
   {ACT_PRINT_DER, 0, 0, 0, ASN1_SUCCESS, __LINE__},
+  {ACT_DELETE, "", "", 0, ASN1_SUCCESS, __LINE__},
   {ACT_CREATE, "TEST_TREE.KrbError", 0, 0, ASN1_SUCCESS, __LINE__},
   {ACT_DECODING, 0, 0, 0, ASN1_SUCCESS, __LINE__},
   {ACT_VISIT, "", "", ASN1_PRINT_ALL, ASN1_SUCCESS, __LINE__},
   {ACT_DELETE_ELEMENT, "pvno", "", 0, ASN1_SUCCESS, __LINE__},
   {ACT_DELETE_ELEMENT, "pvno", "", 0, ASN1_ELEMENT_NOT_FOUND, __LINE__},
-  {ACT_DELETE, "", "", 0, ASN1_SUCCESS, __LINE__},
+  {ACT_DELETE2, "", "", 0, ASN1_SUCCESS, __LINE__},
 
   /* Test: CHOICE */
   {ACT_CREATE, "TEST_TREE.CertTemplate", 0, 0, ASN1_SUCCESS, __LINE__},
@@ -396,11 +399,13 @@ test_type test_array[] = {
   {ACT_ENCODING, "", 0, 24, ASN1_MEM_ERROR, __LINE__},
   {ACT_ENCODING, "", 0, 25, ASN1_SUCCESS, __LINE__},
   {ACT_PRINT_DER, 0, 0, 0, ASN1_SUCCESS, __LINE__},
+  {ACT_DELETE, "", "", 0, ASN1_SUCCESS, __LINE__},
   {ACT_CREATE, "TEST_TREE.Sequence1", 0, 0, ASN1_SUCCESS, __LINE__},
   {ACT_DECODING_ELEMENT, "int2", 0, 0, ASN1_SUCCESS, __LINE__},
   {ACT_VISIT, "", "", ASN1_PRINT_ALL, ASN1_SUCCESS, __LINE__},
   {ACT_READ, "int2", "\x0a", 1, ASN1_SUCCESS, __LINE__},
   {ACT_READ_LENGTH, "int2", NULL, 1, ASN1_MEM_ERROR, __LINE__},
+  {ACT_DELETE, "", "", 0, ASN1_SUCCESS, __LINE__},
   {ACT_CREATE, "TEST_TREE.Sequence1", 0, 0, ASN1_SUCCESS, __LINE__},
   {ACT_DECODING, 0, 0, 0, ASN1_SUCCESS, __LINE__},
   {ACT_DECODING_START_END, "seq.?2", "START", 10, ASN1_SUCCESS, __LINE__},
@@ -531,6 +536,9 @@ main (int argc, char *argv[])
 	case ACT_DELETE:
 	  result = asn1_delete_structure (&asn1_element);
 	  break;
+	case ACT_DELETE2:
+	  result = asn1_delete_structure2 (&asn1_element, ASN1_DELETE_FLAG_ZEROIZE);
+	  break;
 	case ACT_DELETE_ELEMENT:
 	  result = asn1_delete_element (asn1_element, test->par1);
 	  break;
